function c = lsqfun(t,data,model,guess)
% LSQFUN is the least-square fitting function for the experiments in Smart
% Physics Lab. 
%
%   Syntax:     c = lsqfun(t,data,model,guess);
%
%   OUTPUT(S):
%   'c'         n-dimensional array giving result of the set of n parameters 
%               obtained from lsqcurvefit solver.
%               for model = 1, Damped Simple Harmonic Oscillator
%               c(1) = amplitude
%               c(2) = ang. velocity
%               c(3) = phase difference
%               c(4) = damping factor
%               c(5) = offset
%
%               for model = 2, Quadratic Model
%               c(1) = acceleration/2
%               c(2) = initial velocity
%               c(3) = offset
%
%               for any other model, c(1) to c(n) represent the solver
%               results for the unknown parameters
%
%   INPUTS:
%   't'         array of the timestamps 
%   'data'      One dimensional array of data values which will be fitted
%               to the model.
%   'model'     the model to which the solver will attempt to fit the
%               results. Two built-in models are present. 
%               Enter 1 for the Damped Simple Harmonic Oscillator Model.
%               Enter 2 for the Quadratic Model for linear displacements.
%               Enter a string enclosed in apostrophes for any other model
%               e.g. 'c(1)*xdata+c(2)'
%   'guess'     n x 1 matrix. A starting guess for each unknown parameter               
%               Use a best guess or enter any value for each unknown
%               parameter if not sure. 
%
%   WARNING: Be careful if using a model other than the two built-in model
%   functions. Make sure to enter the model function with the correct
%   syntax, constants as c(1),c(2), ... , c(n) and time or independent
%   variable as xdata. Also make sure to enter the correct number of
%   starting guesses for the unknown parameter in the input guess.
%

if model == 1 % damped simple harmonic oscillator model
    myfun = @(c,xdata) c(1)*cos(c(2)*xdata+c(3)).*exp(c(4)*xdata)+c(5);
    if numel(guess) ~= 5
        disp('Error: Please enter 5 values for model = 1.')
        return;
    end
elseif model == 2 % quadratic model 
    myfun = @(c,xdata) c(1)*xdata.^2+c(2)*xdata+c(3);
    if numel(guess) != 3
        disp('Error: Please enter 3 values for model = 2.')
        return;
    end
else
    myfun = str2func(strcat('@(c,xdata)',model));
end

if size(guess,1) == 1 % if some student enters a 1 x n matrix by mistake
    guess = guess';
end

xdata = t;
ydata = data;
[c] = lsqcurvefit(myfun,guess,xdata,ydata);

end